!
!  Written by Leandro Martínez, 2009-2011.
!  Copyright (c) 2009-2018, Leandro Martínez, Jose Mario Martinez,
!  Ernesto G. Birgin.
!
!  Module that contains the temporary data for swap molecules
!  for individual packing
!

module swaptypemod

   integer :: ntemp, ntottemp
   double precision, allocatable :: xfull(:) ! (nn)

end module swaptypemod

